var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};

// source/index.ts
import fetch from "node-fetch";
import { stringify } from "querystring";

// source/utils.ts
var filterTrack = (track) => {
  var _a;
  return {
    title: track == null ? void 0 : track.name,
    artist: (_a = track == null ? void 0 : track.artists) == null ? void 0 : _a.map((artist) => artist == null ? void 0 : artist.name).join(", "),
    link: track == null ? void 0 : track.external_urls.spotify
  };
};
var encodeToBase64 = (str) => Buffer.from(str).toString("base64");

// source/index.ts
var ENDPOINTS = {
  accessToken: "https://accounts.spotify.com/api/token",
  currentlyPlaying: "https://api.spotify.com/v1/me/player/currently-playing",
  lastPlayed: "https://api.spotify.com/v1/me/player/recently-played",
  topItems: "https://api.spotify.com/v1/me/top"
};
var SpotifyClient = class {
  constructor({ clientId, clientSecret, refreshToken }) {
    this.accessToken = null;
    this._genAccesToken = async () => {
      const basicToken = encodeToBase64(`${this.clientId}:${this.clientSecret}`);
      const response = await fetch(ENDPOINTS.accessToken, {
        method: "POST",
        headers: {
          Authorization: `Basic ${basicToken}`,
          "Content-Type": "application/x-www-form-urlencoded"
        },
        body: stringify({
          grant_type: "refresh_token",
          refresh_token: this.refreshToken
        })
      });
      const responseData = await response.json();
      return responseData == null ? void 0 : responseData.access_token;
    };
    this.getCurrentTrack = async ({
      fallbackToLastPlayed = true
    } = {}) => {
      try {
        if (this.accessToken === null)
          this.accessToken = await this._genAccesToken();
        const headers = { Authorization: `Bearer ${this.accessToken}` };
        const response = await fetch(ENDPOINTS.currentlyPlaying, { headers });
        if (response.status === 401) {
          this.accessToken = await this._genAccesToken();
          return this.getCurrentTrack({ fallbackToLastPlayed });
        }
        let isPlaying = true;
        if (response.status === 204) {
          isPlaying = false;
          return fallbackToLastPlayed ? __spreadValues({ isPlaying }, (await this.getRecentTracks())[0]) : null;
        }
        const responseData = await response.json();
        const currentTrack = filterTrack(responseData.item);
        return __spreadValues({ isPlaying }, currentTrack);
      } catch (error) {
        throw new Error(error);
      }
    };
    this.getRecentTracks = async (limit = 1) => {
      try {
        if (this.accessToken === null)
          this.accessToken = await this._genAccesToken();
        if (limit > 50 || limit < 1)
          throw new Error("Limit must be between 1 and 50");
        const headers = { Authorization: `Bearer ${this.accessToken}` };
        const response = await fetch(`${ENDPOINTS.lastPlayed}?limit=${limit}`, { headers });
        if (response.status === 401) {
          this.accessToken = await this._genAccesToken();
          return this.getRecentTracks(limit);
        }
        const responseData = await response.json();
        const lastPlayedTrack = responseData.items.map(({ track }) => filterTrack(track));
        return lastPlayedTrack;
      } catch (error) {
        throw new Error(error);
      }
    };
    this.getTopTracks = async ({
      limit = 10,
      timeRange = "short"
    } = {}) => {
      try {
        if (this.accessToken === null)
          this.accessToken = await this._genAccesToken();
        if (limit > 50 || limit < 1)
          throw new Error("Limit must be between 1 and 50");
        const headers = { Authorization: `Bearer ${this.accessToken}` };
        const params = stringify({ time_range: `${timeRange}_term`, limit });
        const response = await fetch(`${ENDPOINTS.topItems}/tracks/?${params}`, { headers });
        if (response.status === 401) {
          this.accessToken = await this._genAccesToken();
          return this.getTopTracks({ limit, timeRange });
        }
        const responseData = await response.json();
        const topTracks = responseData.items.map((item) => ({
          title: item.name,
          artist: item.artists[0].name,
          link: item.external_urls.spotify
        }));
        return topTracks;
      } catch (error) {
        throw new Error(error);
      }
    };
    this.clientId = clientId;
    this.clientSecret = clientSecret;
    this.refreshToken = refreshToken;
  }
};
export {
  SpotifyClient
};
